@@ -1,4 +1,4 @@
-################################################################################
+############################################################################## #
 # Copyright (C) 2019 by 52 North                                               #
 # Initiative for Geospatial Open Source Software GmbH                          #
 #                                                                              #
@@ -25,28 +25,59 @@
 # Created: 2010-06-18                                                          #
 # Project: sos4R - https://github.com/52North/sos4R                            #
 #                                                                              #
-################################################################################
+############################################################################## #
 
 #
+# parsing an ows:Operation ----
 #
-#
-parseOwsOperation <- function(obj, namespaces = SosAllNamespaces()) {
-  .name <- xml2::xml_attr(x = obj, attr = "name")
+parseOwsOperation <- function(obj, sos) {
+  name <- xml2::xml_attr(x = obj, attr = "name")
 
-  .dcpsXML <- xml2::xml_find_all(x = obj, xpath = owsDCPName, ns = namespaces)
-  .dcps <- list()
-  for (.dcp in .dcpsXML) {
-    .http <- xml2::xml_child(x = .dcp, search = owsHTTPName, ns = namespaces)
-    .endpoints <- xml2::xml_find_all(x = .http, xpath = paste0(owsGetName, "|", owsPostName), ns = namespaces)
+  dcpsXML <- xml2::xml_find_all(x = obj, xpath = owsDCPName, ns = sos@namespaces)
+  dcps <- list()
+  for (dcp in dcpsXML) {
+    http <- xml2::xml_child(x = dcp, search = owsHTTPName, ns = sos@namespaces)
+    endpoints <- xml2::xml_find_all(x = http, xpath = paste0(owsGetName, "|", owsPostName), ns = sos@namespaces)
 
-    for (.ep in .endpoints) {
-      .newEndpoint <- list(xml2::xml_attr(x = .ep, attr = "xlink:href", ns = namespaces))
-      names(.newEndpoint) <- xml2::xml_name(x = .ep, ns = namespaces)
-      .dcps <- c(.dcps, .newEndpoint)
+    for (ep in endpoints) {
+      httpMethod <- xml2::xml_name(x = ep, ns = sos@namespaces)
+      href <- xml2::xml_attr(x = ep, attr = "xlink:href", ns = sos@namespaces)
+
+      constraints <- xml2::xml_find_all(x = ep, xpath = owsConstraintName, ns = sos@namespaces)
+      if (!length(constraints)) {
+        contentTypes <- list(NA)
+      }
+      else {
+        contentTypes <- list()
+
+        for (constraint in constraints) {
+          if (xml2::xml_attr(x = constraint, attr = "name") == owsContentTypeConstraintName) {
+
+            allowedValuesXml <- xml2::xml_find_all(x = constraint,
+                                                   xpath = paste0(owsAllowedValuesName, "/", owsValueName),
+                                                   ns = sos@namespaces)
+            if (length(allowedValuesXml) > 0) {
+              contentTypes <- c(contentTypes, xml2::xml_text(allowedValuesXml))
+            }
+          }
+        }
+
+        if (!length(contentTypes)) contentTypes <- list(NA)
+      }
+
+      for (contentType in contentTypes) {
+        dcp <- list()
+        dcp[[owsDcpHttpMethodIndex]] <- httpMethod
+        dcp[[owsDcpContentTypeIndex]] <- contentType
+        dcp[[owsDcpUrlIndex]] <- href
+        dcps[[length(dcps) + 1]] <- dcp
+      }
     }
   }
 
-  .parametersXML <- xml2::xml_find_all(x = obj, xpath = owsParameterName, ns = namespaces)
+  names(dcps) <- sapply(X = dcps, FUN = function(x) {x[[owsDcpHttpMethodIndex]]})
+
+  .parametersXML <- xml2::xml_find_all(x = obj, xpath = owsParameterName, ns = sos@namespaces)
   .parameters = list()
   .names = list()
 
@@ -55,23 +86,24 @@ parseOwsOperation <- function(obj, namespaces = SosAllNamespaces()) {
       .allowedValuesAndRanges <- list()
 
       # check for ows:AnyValue
-      if (!is.na(xml2::xml_child(x = .p, search = owsAnyValueName, ns = namespaces))) {
+      if (!is.na(xml2::xml_child(x = .p, search = owsAnyValueName, ns = sos@namespaces))) {
         .allowedValuesAndRanges = list(owsAnyValueName)
       }
       else {
         # try list of allowed values
         .allowedValuesXml <- xml2::xml_find_all(x = .p,
                                                 xpath = paste0(owsAllowedValuesName, "/", owsValueName),
-                                                ns = namespaces)
+                                                ns = sos@namespaces)
         if (length(.allowedValuesXml) > 0)
           .allowedValuesAndRanges <- c(.allowedValuesAndRanges, xml2::xml_text(.allowedValuesXml))
 
         # try list of ranges
         .rangesXml <- xml2::xml_find_all(x = .p,
                                          xpath = paste0(owsAllowedValuesName, "/", owsRangeName),
-                                         ns = namespaces)
+                                         ns = sos@namespaces)
         if (length(.rangesXml) > 0)
-          .allowedValuesAndRanges <- c(.allowedValuesAndRanges, sapply(.rangesXml, parseOwsRange))
+          .allowedValuesAndRanges <- c(.allowedValuesAndRanges,
+                                       sapply(.rangesXml, parseOwsRange))
       }
 
       .names <- c(.names, xml2::xml_attr(x =  .p, attr = "name"))
@@ -81,26 +113,26 @@ parseOwsOperation <- function(obj, namespaces = SosAllNamespaces()) {
     names(.parameters) <- .names
   }
 
-  if (any(sapply(names(obj), "==", owsConstraintName)))
-    warning("constraint elements are NOT processed!")
-  .constraints = list(NA)
+  if (length(xml2::xml_find_all(x = obj, xpath = owsConstraintName)))
+    warning("operation level constraint elements are NOT processed!")
+  constraints = list(NA)
 
-  if (any(sapply(names(obj), "==", owsMetadataName)))
-    warning("metadata elements are NOT processed!")
-  .metadata = list(NA)
+  if (length(xml2::xml_find_all(x = obj, xpath = owsMetadataName)))
+    warning("operation level metadata elements are NOT processed!")
+  metadata = list(NA)
 
-  .op <- OwsOperation(name = .name,
-                      DCPs = .dcps,
+  .op <- OwsOperation(name = name,
+                      DCPs = dcps,
                       parameters = .parameters,
-                      constraints = .constraints,
-                      metadata = .metadata)
+                      constraints = constraints,
+                      metadata = metadata)
   return(.op)
 }
 
 #
-# parsing an ows:ExceptionReport ---
+# parsing an ows:ExceptionReport ----
 #
-parseOwsExceptionReport <- function(obj, verbose = FALSE) {
+parseOwsExceptionReport <- function(obj, sos, verbose = FALSE) {
   if (verbose) cat("[parseOwsExceptionReport] Starting ...\n")
   .docRoot <- xml2::xml_root(x = obj)
 
@@ -109,9 +141,9 @@ parseOwsExceptionReport <- function(obj, verbose = FALSE) {
 
   .exceptionsXML <- xml2::xml_find_all(x = .docRoot,
                                        xpath = paste0("//", owsExceptionName),
-                                       ns = SosAllNamespaces())
+                                       ns = sos@namespaces)
 
-  .exceptions = sapply(.exceptionsXML, parseOwsException)
+  .exceptions = sapply(.exceptionsXML, parseOwsException, sos = sos)
   if (verbose) cat("[parseOwsExceptionReport]", length(.exceptions), "exceptions.")
 
   .report <- OwsExceptionReport(version = .version, lang = .lang, exceptions = .exceptions)
@@ -122,13 +154,13 @@ parseOwsExceptionReport <- function(obj, verbose = FALSE) {
 #
 # parsing an ows:Exception ----
 #
-parseOwsException <- function(obj) {
+parseOwsException <- function(obj, sos) {
   .code <- xml2::xml_attr(x = obj, attr = "exceptionCode")
   .locator <- xml2::xml_attr(x = obj, attr = "locator", default = NA_character_)
 
   .text <- xml2::xml_find_all(x = obj,
                               xpath = owsExceptionTextName,
-                              ns = SosAllNamespaces())
+                              ns = sos@namespaces)
 
   if (!is.na(.text))
     .text <- xml2::xml_text(x = .text)
@@ -142,34 +174,34 @@ parseOwsException <- function(obj) {
 }
 
 #
+# parsing ows:ServiceIdentification ----
 #
-#
-parseOwsServiceIdentification <- function(obj, namespaces = SosAllNamespaces()) {
+parseOwsServiceIdentification <- function(obj, sos) {
   .serviceType <- xml2::xml_text(xml2::xml_child(x = obj,
                                                  search = owsServiceTypeName,
-                                                 ns = namespaces))
+                                                 ns = sos@namespaces))
   .serviceTypeVersion <- xml2::xml_text(xml2::xml_child(x = obj,
                                                         search = owsServiceTypeVersionName,
-                                                        ns = namespaces))
+                                                        ns = sos@namespaces))
   .title <- xml2::xml_text(xml2::xml_child(x = obj,
                                            search = owsTitleName,
-                                           ns = namespaces))
+                                           ns = sos@namespaces))
 
   # optional:
-  .profile <- xml2::xml_text(xml2::xml_find_all(x = obj, xpath = owsProfileName, ns = namespaces))
+  .profile <- xml2::xml_text(xml2::xml_find_all(x = obj, xpath = owsProfileName, ns = sos@namespaces))
 
-  .abstract <- xml2::xml_text(xml2::xml_find_all(x = obj, xpath = owsAbstractName, ns = namespaces))
+  .abstract <- xml2::xml_text(xml2::xml_find_all(x = obj, xpath = owsAbstractName, ns = sos@namespaces))
 
   .keywords <- c(NA)
-  .keywordsList <- xml2::xml_child(x = obj, search = owsKeywordsName, ns = namespaces)
+  .keywordsList <- xml2::xml_child(x = obj, search = owsKeywordsName, ns = sos@namespaces)
   if (!is.na(.keywordsList))
     .keywords <- xml2::xml_text(xml2::xml_find_all(x = .keywordsList,
                                                    xpath = owsKeywordName,
-                                                   ns = namespaces))
+                                                   ns = sos@namespaces))
 
-  .fees <- xml2::xml_text(xml2::xml_find_all(x = obj, xpath = owsFeesName, ns = namespaces))
+  .fees <- xml2::xml_text(xml2::xml_find_all(x = obj, xpath = owsFeesName, ns = sos@namespaces))
 
-  .accessConstraints <- xml2::xml_text(xml2::xml_find_all(x = obj, xpath = owsAccessConstraintsName, ns = namespaces))
+  .accessConstraints <- xml2::xml_text(xml2::xml_find_all(x = obj, xpath = owsAccessConstraintsName, ns = sos@namespaces))
 
   .si <- OwsServiceIdentification(serviceType =  .serviceType,
                                   serviceTypeVersion = .serviceTypeVersion,
@@ -183,20 +215,20 @@ parseOwsServiceIdentification <- function(obj, namespaces = SosAllNamespaces())
 }
 
 #
+# parsing ows:ServiceProvider ----
 #
-#
-parseOwsServiceProvider <- function(obj) {
+parseOwsServiceProvider <- function(obj, sos) {
   .name <- xml2::xml_text(x = xml2::xml_child(x = obj,
                                               search = owsProviderNameName,
-                                              ns = SosAllNamespaces()))
+                                              ns = sos@namespaces))
 
   # optional:
-  .site <- xml2::xml_child(x = obj, search = owsProviderSiteName, ns = SosAllNamespaces())
+  .site <- xml2::xml_child(x = obj, search = owsProviderSiteName, ns = sos@namespaces)
   if (!is.na(.site))
     .site <- xml2::xml_attr(x = .site, attr = "href", default = NA_character_)
   else .site <- as.character(NA)
 
-  .contact <- xml2::xml_child(x = obj, search = owsServiceContactName, ns = SosAllNamespaces())
+  .contact <- xml2::xml_child(x = obj, search = owsServiceContactName, ns = sos@namespaces)
   if (!is.na(.contact)) {
     .sp <- OwsServiceProvider(providerName = .name,
                               providerSite = .site,
@@ -208,6 +240,7 @@ parseOwsServiceProvider <- function(obj) {
 }
 
 #
+# parsing ows:Range ----
 # all elements are optional
 #
 parseOwsRange <- function(obj) {
